package net.sf.sido.factory;

import net.sf.sido.DataObject;
import net.sf.sido.DataSchema;
import net.sf.sido.factory.json.JSONDataFactoryReader;
import net.sf.sido.factory.sido.SiDOLDataFactoryReader;
import net.sf.sido.factory.support.AbstractDataFactoryReader;
import net.sf.sido.factory.support.SiDOSchema;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

/**
 * This DataFactoryReader reads URLs and delegates the loading to 
 */
public class UniversalDataFactoryReader extends AbstractDataFactoryReader {

	private final Logger logger = LoggerFactory
			.getLogger(UniversalDataFactoryReader.class);

	private Map<String, DataFactoryReader> readers = new HashMap<String, DataFactoryReader>();

	public UniversalDataFactoryReader() {
		readers.put("json", new JSONDataFactoryReader());
		readers.put("sidol", new SiDOLDataFactoryReader());
	}

    public Map<String, DataFactoryReader> getReaders() {
        return readers;
    }

    public void setReaders(Map<String, DataFactoryReader> readers) {
        this.readers = readers;
    }

    @Override
    public DataObject readSchema(SiDOSchema sidoSchema, URL url) throws IOException {
        // Resource name
        String name = url.getFile();
        // Gets the extension
        String extension = StringUtils.substringAfterLast(name, ".");
        if (StringUtils.isBlank(extension)) {
            throw new IOException(
                    String.format(
                            "Cannot load %s because no file extension is available to determine the loader type.",
                            url));
        }
        extension = StringUtils.lowerCase(extension);
        logger.debug("Resource extension is [{}]", extension);
        // Gets the associated loader
        DataFactoryReader reader = readers.get(extension);
        if (reader == null) {
            throw new IOException(
                    String.format("Cannot find any DataFactoryReader for extension %s. Please register a reader for this extension.", extension));
        }
        logger.debug("Using reader {}", reader);
        // Reads the schema
        logger.debug("Reading the schema from URL {}", url);
        return reader.readSchema(sidoSchema, url);
    }
}
