package net.sf.sido.io;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import net.sf.sido.DataFactory;
import net.sf.sido.DataObject;
import net.sf.sido.DataType;

public final class DataObjectIO {

	public static final String ENCODING = "UTF-8";

	public static DataObject readString(String value, DataObjectReader<?> reader, DataFactory factory) throws IOException {
		return readString(value, reader, factory, null);
	}

	public static DataObject readString(String value, DataObjectReader<?> reader, DataFactory factory, DataType expectedType) throws IOException {
		BufferedInputStream in = new BufferedInputStream(new ByteArrayInputStream(value.getBytes(ENCODING)));
		try {
			return reader.read(in, factory, expectedType);
		} finally {
			in.close();
		}
	}

	public static String writeString(DataObjectWriter<?, ?> writer, DataObject o) throws IOException {
		return writeString(writer, o, DOReferenceManagement.STACK);
	}

	public static String writeString(DataObjectWriter<?, ?> writer, DataObject o, DOReferenceManagement referenceManagement) throws IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		BufferedOutputStream bout = new BufferedOutputStream(out);
		try {
			writer.write(bout, o, referenceManagement);
		} finally {
			bout.close();
		}
		return out.toString(ENCODING);
	}

	private DataObjectIO() {
	}

}
