package net.sf.sido.support;

import java.util.Map;

import net.sf.sido.DOInvalidPropertyTypeException;
import net.sf.sido.DataObject;
import net.sf.sido.Property;
import net.sf.sido.PropertyController;
import net.sf.sido.PropertyMetaInfos;
import net.sf.sido.array.Array;

public class ArrayPropertyController<T> implements PropertyController<Array<T>> {

	private final Class<T> elementType;

	public ArrayPropertyController(Class<T> elementType) {
		this.elementType = elementType;
	}

	@Override
	public Array<T> getDefaultValue(boolean deep) {
		return Array.empty();
	}

	@Override
	public void control(Property<Array<T>> property, DataObject o, Array<T> value) {
		if (value != null) {
			for(T t : value) {
				if (t != null && !this.elementType.isAssignableFrom(t.getClass())) {
					throw new DOInvalidPropertyTypeException(o, property.getName(), this.elementType.getName(), value);
				}
			}
		}
	}

	@Override
	public void fillMetaInfo(Map<String, Object> mi) {
		mi.put(PropertyMetaInfos.ARRAY, true);
		mi.put(PropertyMetaInfos.ELEMENT_TYPE, this.elementType);
	}
}
