package net.sf.sido.support;

import net.sf.sido.*;
import net.sf.sido.array.IndexedArray;
import org.apache.commons.lang.StringUtils;

import java.util.Map;

public class IndexedArrayDOPropertyController<K> implements PropertyController<IndexedArray<DataObject, K>> {

    private final DataType elementType;
    private final String indexProperty;

    public IndexedArrayDOPropertyController(DataType elementType, String indexProperty) {
        this.elementType = elementType;
        this.indexProperty = indexProperty;
        // Controls the indexed property
        controlIndex(elementType, indexProperty);
    }

    protected void controlIndex(DataType elementType, String indexProperty) {
		// Gets the property of the type
		Property<Object> property = elementType.getProperty(indexProperty);
		if (property == null) {
			throw new DOUndefinedPropertyException(elementType, indexProperty);
		}
		// Checks it is a simple property
		if (!property.isSimple()) {
			throw new DOIndexedArrayIndexNotSimpleException(elementType, indexProperty);
		}
		// Checks it is not an array
		if (property.isArray()) {
			throw new DOIndexedArrayIndexArrayException(elementType, indexProperty);
		}
		// Checks it is not nullable
		if (property.isNullable()) {
			throw new DOIndexedArrayIndexNullableException(elementType, indexProperty);
		}
    }

    @Override
    public IndexedArray<DataObject, K> getDefaultValue(boolean deep) {
        return new IndexedArray<DataObject, K>(DOFunctions.<K>get(indexProperty));
    }

    @Override
    public void control(Property<IndexedArray<DataObject, K>> property, DataObject o, IndexedArray<DataObject, K> value) {
        if (value != null) {
            for (DataObject t : value) {
                if (t != null && !t.getSidoType().isSubClass(elementType)) {
                    throw new DOInvalidPropertyTypeException(o, property.getName(), elementType.getQualifiedName(), value);
                }
            }
        }
    }

    @Override
    public void fillMetaInfo(Map<String, Object> mi) {
        mi.put(PropertyMetaInfos.ARRAY, true);
        mi.put(PropertyMetaInfos.ARRAY_INDEX, true);
        mi.put(PropertyMetaInfos.ARRAY_INDEX_PROPERTY, indexProperty);
        mi.put(PropertyMetaInfos.DO, true);
        mi.put(PropertyMetaInfos.ELEMENT_TYPE, elementType);
    }
}
