package net.sf.sido.support;

import java.util.Arrays;
import java.util.List;

import net.sf.sido.PropertyAdapter;

public class MultiplePropertyAdapter<T> implements PropertyAdapter<T> {

	private final List<? extends PropertyAdapter<T>> adapters;

	public MultiplePropertyAdapter(List<? extends PropertyAdapter<T>> adapters) {
		this.adapters = adapters;
	}

	public MultiplePropertyAdapter(PropertyAdapter<T>... adapters) {
		this.adapters = Arrays.asList(adapters);
	}

	@Override
	public boolean canConvert(Object o) {
		for(PropertyAdapter<T> adapter : adapters) {
			if (adapter.canConvert(o)) {
				return true;
			}
		}
		return false;
	}

	@Override
	public T convert(Object o) {
		for(PropertyAdapter<T> adapter : adapters) {
			if (adapter.canConvert(o)) {
				return adapter.convert(o);
			}
		}
		throw new IllegalArgumentException("Could not find any adapter for " + o);
	}

}
