package net.sf.sido.support;

import java.lang.reflect.Method;

import net.sf.sido.PropertyAdapter;

public class NumericPropertyAdapter<T extends Number> implements PropertyAdapter<T> {

	private final Method conversionMethod;

	public NumericPropertyAdapter(String conversionMethod) {
		try {
			this.conversionMethod = Number.class.getMethod(conversionMethod);
		} catch (Exception ex) {
			throw new IllegalArgumentException("Cannot get method " + conversionMethod + " on Number class", ex);
		}
	}

	@Override
	public boolean canConvert(Object o) {
		return (o instanceof Number);
	}

	@SuppressWarnings("unchecked")
	@Override
	public T convert(Object o) {
		if (o instanceof Number) {
			try {
				return (T) conversionMethod.invoke(o);
			} catch (Exception e) {
				throw new RuntimeException("Cannot convert " + o, e);
			}
		} else {
			return (T) o;
		}
	}
}
